August 1, 2002:
-------------------------------------------------------------------
Considering that the only stable version to date was the one with both
the Tcl and Tk event loops replacing the magic Tx command and event
loops, respectively, I am reverting (mostly) to that version, wherein
no threading was used except for the DRC function.  **However**, I
want to expand on that concept in three ways:

1) Get the OpenGL interface running, which hopefully should work in
   the non-threaded environment

2) Do whatever it takes to get the DRC to work as an "idle callback"
   and NOT as a separate thread.

3) Do whatever it takes to get character-event-based stdin/stdout
   handling in Tcl.  Either this is relatively simple (play with
   fcntl calls on stdin and put it in non-canonical mode with no
   line buffering) or this is relatively difficult (create a new
   handler type in Tcl with all the associated open/close/read/
   write routines).

-------------------------------------------------------------------
Magic Toolscript---A Tcl-wrapped version of magic
-------------------------------------------------------------------

-------------------------------------------------------------------
Compiling and Installation
-------------------------------------------------------------------

Simplified methods for compiling and installing have not been introduced yet,
so follow these instructions.

1) You will need "swig" and Tcl version 8+.  Testing has been done on swig
   versions 1.1 and 1.3.10.  Testing has been done on Tcl and Tk versions
   8.3, but lower versions should suffice.  HOWEVER, Tcl and Tk MUST have
   been compiled with "configure --enable-threads".  If not, the system will
   hang after doing "magic::start".  Note that most distributions come with
   non-thread compliant precompiled Tcl and Tk.

2) At the top level, do "make config" as usual.  Choose "Tk(Tcl)" as the
   graphics option.  Readline, Scheme, and Sim modules will be disabled
   automatically.

3) Do "make wrapper".  This will run "swig" and should complete the compilation 
   of the Tcl-compatible shared library file "magic/magicwrap.so".

4) Hand-edit magic/magic.sh and replace the HOME directory with the proper
   location of the source.  Note that this is supposed to fool wish into
   running the contents of ".wishrc" from this directory on startup.  It
   is the only way to get wish to run both interactively and also process
   some commands on startup.  On install, "magic/magic.sh" should go to
   /home/cad/bin/magic.sh or /home/cad/bin/magic if you intend to use it
   as the sole version of magic on the system; "magic/.wishrc" should go
   into /home/cad/lib/magic/tcl/.wishrc, and the HOME variable in "magic.sh"
   should then point to directory "/home/cad/lib/magic/tcl".

   The path to wish should be edited in magic.sh to match what is on the
   system.

5) ".tcl" files (packages) go in /home/cad/lib/magic/tcl/.

6) The startup script works out namespaces so that all magic commands
   which do not conflict with existing tcl commands become directly available
   from the command line.  Otherwise, all magic commands are prefixed by the 
   namespace "::magic::" to differentiate them from other applications.
   "pushing" the namespace is not possible in any real sense in Tcl (I don't
   know why; the need for it seems obvious enough), but the commands
   (defined in the .wishrc startup file) "pushnamespace" and "popnamespace"
   do about 95% of it.

7) Make sure that the default .magic script (installed) does not have embedded
   control characters (see below in "Known Bugs").

8) magic-toolscript should work with "tclreadline".

9) Note that the "imacro" (interactive macro) function works by sending
   events to the terminal window.  This is only possible if the terminal
   window allows "send event"-type events.  Globally, it can be allowed
   by setting "XTerm: allowSendEvents" in the .Xdefaults file.  However,
   because this is inherently insecure, it is probably a better idea to
   select the "Main Options" menu in the xterm (Control-Mouse button 1)
   and select "Allow SendEvents".

   It is intended that magic-toolscript will allow a Tk console window
   (such as TkCon) to take over the functionality of the terminal, in
   which case graphics/grX11su1.c will be rewritten to allow smooth
   switching between the console and the magic window.

-------------------------------------------------------------------
Known Bugs and/or Annoyances
-------------------------------------------------------------------

1) Commands "list", "split", "array", "source", "flush", and "load"
   all conflict with Tcl commands.  Either my implementation of Tcl
   is broken, or else Tcl's "rename" command doesn't work as advertised,
   because what I want to do is to rename the unqualified Tcl command
   "list" as the magic command, and leave the qualified Tcl name "::list"
   as the Tcl version of list.  This would effectively act like a true
   namespace push/pop.  Instead, I do "rename list {}" and get
   "self-referential recursion in "unknown" for command "list""
   instead of "invalid command name "list"".

   Current implementation is to require that these command names be
   fully-qualified;  e.g., "::magic::load" or "magic::load".  The
   unqualified names (e.g., "::load" or "load") refer to the Tcl function.

2) The Tcl prompt or a " >" prompt
   appears at all times.  The desired behavior is that the magic prompt
   ">" (or "]" while DRC is pending) should appear when the cursor is
   in a window, indicating that the input is accepting keyboard macros.
   When outside of the window or after using the macro ":" to get to the
   command-line entry from magic, the Tcl "%" prompt appears, indicating
   that all valid Tcl commands may be entered at the prompt.

   Because it is not possible to detect if a partial Tcl command is on
   the command line, it is only possible to enter part of a command in
   the terminal window and complete the command in the magic window if
   ":" is typed in the magic window to redirect keyboard input to the
   terminal.

3) The "*" prefixing magic "wizard" commands is a bad idea and should be
   changed to something else.  While the "pushnamespace/popnamespace"
   commands deal with it properly, it is likely to be a perennial gotcha,
   and mishandling can cause atrocious things to happen.

4) This appears to be another Tcl bug, but if the same user is running
   Tcl in another window, magic will segmentation fault on startup.
   Have not tried to track this one down.

5) Magic's original ".magic" file has control characters embedded.  This
   is the only magic command syntax which is illegal Tcl syntax.  However,
   magic 7.1 allows control characters to be written either as "^X" (two
   characters, with a caret) or "Control_X".  If the original .magic is
   used (i.e., magic-toolscript is compiled and run, but not installed),
   the embedded controls should be changed to printable ASCII format.

-------------------------------------------------------------------
