/* magicwrap.i --- SWIG directives for creating the interpreter-wrapped	*/
/* version of magic.							*/

%module magicwrap

%typemap(tcl8, ignore) int argc(int n_arg) {
    $1 = n_arg = objc - 1;
    objc = 2;
}

%typemap(tcl8, in) char **argv {
    int templength, i;
    $1 = (char **) mallocMagic(n_arg1 * sizeof(char *));
    for (i = 0; i < n_arg1; i++) {
	$1[i] = Tcl_GetStringFromObj(objv[i + 1], &templength);
    }
}

%typemap(tcl8, freearg) char **argv {
    freeMagic($1);
}

%{
#include <tk.h>

#include <unistd.h>
#include <signal.h>

#include "misc/magic.h"   
#include "utils/geometry.h"
#include "tiles/tile.h"  
#include "utils/hash.h"  
#include "utils/dqueue.h"
#include "database/database.h"
#include "windows/windows.h"
#include "commands/commands.h"
#include "utils/utils.h"
#include "textio/textio.h"
#include "textio/txcommands.h"
#include "signals/signals.h"
#include "graphics/graphics.h"
#include "utils/malloc.h" 

/*
 * String containing the version number of magic.  Don't change the string
 * here, nor its format.  It is updated by the Makefile in this directory.
 */

char *MagicVersion = MAGIC_VERSION;
char *MagicCompileTime = MAGIC_DATE;

Tcl_Interp *magicinterp;

/* Forward declaration */

int TerminalInputProc(ClientData, char *, int, int *);

/*----------------------------------------*/
/* Inline the two main wrapper prodedures */
/*----------------------------------------*/
 
static int _magic_start(ClientData clientData,
        Tcl_Interp *interp, int argc, char *argv[])
{
    int result;
    Tcl_ChannelType *inChannel;
    Tcl_Obj *consoleName;

    /* Remember the interpreter */
    magicinterp = interp;

    fprintf(stderr, "Starting magic under Tcl interpreter\n");
 
    mainInitBeforeArgs(argc, argv);
    mainDoArgs(argc, argv);
    mainInitAfterArgs();

    TxResetTerminal();

    /* Set initial command structure */

    TxCurCommand.tx_p.p_x = 0;
    TxCurCommand.tx_p.p_y = 0;
    TxCurCommand.tx_wid = WIND_UNKNOWN_WINDOW;

    /* Look for Tcl variable "MagicConsole".  If it exists, use the	*/
    /* window by that name for input/output.				*/

    if ((consoleName = Tcl_GetVar(interp, "MagicConsole", TCL_GLOBAL_ONLY)) != NULL)
    {
	/* ... To be finished */
    }
    else
    {
	/* Otherwise, we use the terminal.	*/
	/* Replace the input proc for stdin with our own. */

	inChannel = Tcl_GetChannelType(Tcl_GetStdChannel(TCL_STDIN));
	inChannel->inputProc = TerminalInputProc;
    }

    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* Dispatch a command from Tcl					*/
/*    Put the command onto Tcl's queue.  TxDispatch will take	*/
/*    care of it.						*/
/*--------------------------------------------------------------*/

static int _tcl_dispatch(ClientData clientData,
        Tcl_Interp *interp, int argc, char *argv[])
{
    TxTclDispatch(clientData, argc, argv);
    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* Dispatch a window-related command.  The first argument is	*/
/* the window to which the command should be directed, so we	*/
/* determine which window this is, set "TxCurCommand" values	*/
/* to point to the window, then dispatch the command.		*/
/*--------------------------------------------------------------*/

static int _tk_dispatch(ClientData clientData,
        Tcl_Interp *interp, int argc, char *argv[])
{
    int id;

    if (GrWindowIdPtr)
    {
	id = (*GrWindowIdPtr)(argv[0]);

	/* Find the bottommost point of the window? */
	/* Probably not necessary.  Need to clear the scrollbar, though. */
	TxSetPoint(20, 20, id);
	argc--;
	argv++;
    }
    TxTclDispatch(clientData, argc, argv);
    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* Set up a window to use commands via _tk_dispatch		*/
/*--------------------------------------------------------------*/

void
MakeWindowCommand(char *wname)
{
    Tcl_CreateCommand(magicinterp, wname, _tk_dispatch,
		(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
}

/*--------------------------------------------------------------*/
/* Get a line from stdin (Tcl replacement for Tx function)	*/
/*--------------------------------------------------------------*/

char *
TxGetLine(dest, maxChars)
    char *dest;
    int maxChars;
{
    Tcl_Obj *objPtr;
    int charsStored, length;
    char *string;

    TxFlush();

    objPtr = Tcl_NewObj();
    charsStored = Tcl_GetsObj(Tcl_GetStdChannel(TCL_STDIN), objPtr);
    if (charsStored > 0) {
	string = Tcl_GetStringFromObj(objPtr, &length);
	if (length > maxChars) length = maxChars;
	strcpy(dest, string);
    }
    Tcl_DecrRefCount(objPtr);
    return dest;
}

/*--------------------------------------------------------------*/
/* Parse a file.  This is a skeleton version of the TxDispatch	*/
/* routine in textio/txCommands.c				*/
/*--------------------------------------------------------------*/

void
TxDispatch(f)
    FILE *f;	/* Under Tcl, we never call this with NULL */
{
    if (f == NULL)
    {
	fprintf(stderr, "Error:  TxDispatch(NULL) was called\n");
    }
    while (!feof(f))
    {
	if (SigInterruptPending)
	{
	    fprintf(stderr, "Read-in of file aborted.\n");
	    fflush(stderr);
	    SigInterruptPending = FALSE;
	    return;
	}
	txGetFileCommand(f, NULL);
    }
}

/*--------------------------------------------------------------*/
/* Send a command line which was collected by magic's TxEvent	*/
/* handler to the interpreter's event queue rather than sending	*/
/* it to TxParseString().					*/
/*--------------------------------------------------------------*/

void
TxParseString(str, q, event)
    char *str;
    caddr_t q;		/* unused */
    caddr_t event;	/* always NULL (ignored) */
{
    char *reply;

    Tcl_Eval(magicinterp, str);

    reply = Tcl_GetStringResult(magicinterp);
    if (strlen(reply) > 0)
	fprintf(stderr, "%s\n", reply);
}

/*--------------------------------------------------------------*/
/* Provide input to Tcl from outside the terminal window by	*/
/* stacking the "stdin" channel.				*/
/*--------------------------------------------------------------*/

typedef struct {
    Tcl_Channel channel;	/* This is all the info we need */
    int fd;
} FileState;

/*--------------------------------------------------------------*/

int
TerminalInputProc(instanceData, buf, toRead, errorCodePtr)
    ClientData instanceData;
    char *buf;
    int toRead;
    int *errorCodePtr;
{
    FileState *fsPtr = (FileState *) instanceData;
    int bytesRead, i, tlen;
    char *locbuf;

    *errorCodePtr = 0;

    TxInputRedirect = False;
    if (TxBuffer != NULL) {
       tlen = strlen(TxBuffer);
       if (tlen < toRead) {
          strcpy(buf, TxBuffer);
	  Tcl_Free(TxBuffer);
	  TxBuffer = NULL;
	  return tlen;
       }
       else {
	  strncpy(buf, TxBuffer, toRead);
	  locbuf = Tcl_Alloc(tlen - toRead + 1);
	  strcpy(locbuf, TxBuffer + toRead);
	  Tcl_Free(TxBuffer);
	  TxBuffer = locbuf;
	  return toRead;
       }
    }

    bytesRead = read(fsPtr->fd, buf, (size_t) toRead);
    if (bytesRead > -1)
	return bytesRead;

    *errorCodePtr = errno;
    return -1;
}

/*--------------------------------------------------------------*/

%}

%init %{
{
    extern char *windCommands[];
    int n;
    char keyword[100];
    char *kwptr = keyword + 7;

    /* Use namespace to avoid conflicts with existing commands */
    strcpy(keyword, "magic::");

    for (n = 0; CmdLongCommands[n] != NULL; n++)
    {
	sscanf(CmdLongCommands[n], "%s ", kwptr); /* get first word */
	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
    }
    for (n = 0; windCommands[n] != NULL; n++)
    {
	sscanf(windCommands[n], "%s ", kwptr); /* get first word */
	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
    }

    /* Start command */
    Tcl_CreateCommand(interp, "magic::start", _magic_start,
			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);

    /* Add the magic TCL directory to the Tcl library search path */

    Tcl_Eval(interp, "lappend auto_path " TCL_DIR );

    /* Export the namespace commands */

    Tcl_Eval(interp, "namespace eval magic namespace export *");
}
%}

